"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const TaskLogger_1 = require("../logger/TaskLogger");
const findNpmClient_1 = require("../workspace/findNpmClient");
const abortSignal_1 = require("./abortSignal");
const path_1 = __importDefault(require("path"));
const formatDuration_1 = require("../logger/reporters/formatDuration");
const getNpmCommand_1 = require("./getNpmCommand");
const workerQueue_1 = require("./workerQueue");
class DistributedNpmScriptTask {
    constructor(task, root, info, config, context, taskDeps) {
        this.task = task;
        this.root = root;
        this.info = info;
        this.config = config;
        this.context = context;
        this.taskDeps = taskDeps;
        this.npmArgs = [];
        this.startTime = [0, 0];
        this.duration = [0, 0];
        DistributedNpmScriptTask.npmCmd = DistributedNpmScriptTask.npmCmd || findNpmClient_1.findNpmClient(config.npmClient);
        this.status = "pending";
        this.logger = new TaskLogger_1.TaskLogger(info.name, task);
        this.npmArgs = getNpmCommand_1.getNpmCommand(config.nodeArgs, config.passThroughArgs, task);
    }
    static killAllActiveProcesses() {
        // first, send SIGTERM everywhere
        for (const cp of DistributedNpmScriptTask.activeProcesses) {
            cp.kill("SIGTERM");
        }
        // wait for "gracefulKillTimeout" to make sure everything is terminated via SIGKILL
        setTimeout(() => {
            for (const cp of DistributedNpmScriptTask.activeProcesses) {
                if (!cp.killed) {
                    cp.kill("SIGKILL");
                }
            }
        }, DistributedNpmScriptTask.gracefulKillTimeout);
    }
    onStart() {
        this.status = "started";
        this.startTime = process.hrtime();
        this.logger.info("started", { status: "started" });
    }
    onComplete() {
        this.status = "completed";
        this.duration = process.hrtime(this.startTime);
        this.logger.info("completed", {
            status: "completed",
            duration: formatDuration_1.hrToSeconds(this.duration),
        });
    }
    onFail() {
        this.status = "failed";
        this.duration = process.hrtime(this.startTime);
        this.logger.info("failed", {
            status: "failed",
            duration: formatDuration_1.hrToSeconds(this.duration),
        });
    }
    onSkipped(hash) {
        this.status = "skipped";
        this.duration = process.hrtime(this.startTime);
        this.logger.info(`skipped`, {
            status: "skipped",
            duration: formatDuration_1.hrToSeconds(this.duration),
            hash,
        });
    }
    runScript() {
        const { info, logger, npmArgs } = this;
        const { npmCmd } = DistributedNpmScriptTask;
        return new Promise((resolve, reject) => {
            const job = workerQueue_1.workerQueue.createJob({
                npmCmd,
                npmArgs,
                name: info.name,
                task: this.task,
                packagePath: path_1.default.relative(this.root, path_1.default.dirname(info.packageJsonPath)),
                taskDeps: getTaskDepsForPackageTask(`${info.name}#${this.task}`, this.taskDeps),
                spawnOptions: {
                    stdio: "pipe",
                    env: Object.assign(Object.assign(Object.assign({}, process.env), (process.stdout.isTTY && this.config.reporter !== "json" && { FORCE_COLOR: "1" })), { LAGE_PACKAGE_NAME: info.name }),
                },
            });
            job.on("succeeded", (result) => {
                logger.info('succeeded');
                resolve();
            });
            job.on("failed", (result) => {
                logger.info('failed');
                reject();
            });
            // times out at 1 hour
            job.timeout(1000 * 60 * 60).save().then((newJob) => {
                logger.info(`job id: ${newJob.id}`);
            });
        });
    }
    async run() {
        const { info, task, context, config, logger } = this;
        try {
            this.onStart();
            await context.profiler.run(() => this.runScript(), `${info.name}.${task}`);
            this.onComplete();
        }
        catch (e) {
            context.measures.failedTasks = context.measures.failedTasks || [];
            context.measures.failedTasks.push({ pkg: info.name, task });
            this.onFail();
            if (config.continueOnError) {
                return true;
            }
            if (!config.safeExit) {
                abortSignal_1.controller.abort();
            }
            return false;
        }
        return true;
    }
}
exports.DistributedNpmScriptTask = DistributedNpmScriptTask;
DistributedNpmScriptTask.npmCmd = "";
DistributedNpmScriptTask.activeProcesses = new Set();
DistributedNpmScriptTask.gracefulKillTimeout = 2500;
function getTaskDepsForPackageTask(packageTask, taskDeps) {
    const stack = [packageTask];
    const deps = new Set();
    const visited = new Set();
    while (stack.length > 0) {
        const current = stack.pop();
        if (visited.has(current)) {
            continue;
        }
        visited.add(current);
        if (current !== packageTask) {
            deps.add(current);
        }
        taskDeps.forEach(([from, to]) => {
            if (to === current) {
                stack.push(from);
            }
        });
    }
    return [...deps];
}
