import { Config } from "../types/Config";
import { PackageInfos } from "workspace-tools";
import { RunContext } from "../types/RunContext";
import { PipelineTarget, TargetConfig, TargetConfigFactory } from "../types/PipelineDefinition";
import { TopologicalGraph } from "../types/TopologicalGraph";
import { Workspace } from "../types/Workspace";
export declare const START_TARGET_ID = "__start";
/**
 * The Pipeline class is the heart of lage's ability to parse a set of target (shorthand, full target config, or even a factory of configs)
 *
 * Some notes:
 * - given a workspace (package infos) and config (merged CLI and config from file), it'll then parse the config.pipeline
 * - the pipeline configuration is defined in the types/PipelineDefinition.ts file
 * - loading config means that for each of the config.pipeline key-value, it'll try to convert them into a set of PipelineTarget
 * - PipelineTargets are objects that hold enough info for an execution engine to run the target function (provides info about priority, cache output globs, etc.)
 * - for the shorthand (e.g. {build: ['^build']}), it'll assume to the PipelineTarget is of "package" type & that the run functions are npm scripts
 * - for a target config, it'll default to "package" and create a PipelineTarget
 * - for a target config factory function, it'll run that function and expect a list of target configs as a result. Those will be converted to a list of PipelineTarget's
 * - after creating the pipeline targets, it'll determine the dependencies (the edges) from one target to another
 * - a pipeline "run(context)" call will provide some "entry targets", and then a subgraph will be generated to be submitted to the execution engine (p-graph)
 *
 * Distributed notes:
 * - for doing distributed work, the WrapperTask will instead place the PipelineTarget info into a worker queue
 */
export declare class Pipeline {
    private workspace;
    private config;
    targets: Map<string, PipelineTarget>;
    dependencies: [string, string][];
    packageInfos: PackageInfos;
    graph: TopologicalGraph;
    constructor(workspace: Workspace, config: Config);
    private generatePackageTarget;
    /**
     * Expands the shorthand notation to pipeline targets (executable units)
     */
    private expandShorthandTargets;
    /**
     * Given an id & factory, generate targets configurations
     * @param id
     * @param factory
     */
    private generateFactoryTargets;
    /**
     * Converts target configuration to pipeline target
     * @param id
     * @param target
     */
    private convertToPipelineTarget;
    /**
     * Adds a target definition (takes in shorthand, target config, or a target config factory)
     * @param id
     * @param targetDefinition
     */
    addTargetDefinition(id: string, targetDefinition: string[] | TargetConfig | TargetConfigFactory): void;
    addDependencies(): void;
    generateTargetGraph(): [string, string][];
    loadConfig(config: Config): void;
    private getTargetPriority;
    run(context: RunContext): Promise<void>;
}
