"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Pipeline = exports.START_TARGET_ID = void 0;
const generateTopologicalGraph_1 = require("../workspace/generateTopologicalGraph");
const NpmScriptTask_1 = require("./NpmScriptTask");
const p_graph_1 = __importDefault(require("p-graph"));
const path_1 = __importDefault(require("path"));
const getPipelinePackages_1 = require("./getPipelinePackages");
const taskId_1 = require("./taskId");
const WrappedTarget_1 = require("./WrappedTarget");
const DistributedTarget_1 = require("./DistributedTarget");
const workerQueue_1 = require("./workerQueue");
exports.START_TARGET_ID = "__start";
/**
 * The Pipeline class is the heart of lage's ability to parse a set of target (shorthand, full target config, or even a factory of configs)
 *
 * Some notes:
 * - given a workspace (package infos) and config (merged CLI and config from file), it'll then parse the config.pipeline
 * - the pipeline configuration is defined in the types/PipelineDefinition.ts file
 * - loading config means that for each of the config.pipeline key-value, it'll try to convert them into a set of PipelineTarget
 * - PipelineTargets are objects that hold enough info for an execution engine to run the target function (provides info about priority, cache output globs, etc.)
 * - for the shorthand (e.g. {build: ['^build']}), it'll assume to the PipelineTarget is of "package" type & that the run functions are npm scripts
 * - for a target config, it'll default to "package" and create a PipelineTarget
 * - for a target config factory function, it'll run that function and expect a list of target configs as a result. Those will be converted to a list of PipelineTarget's
 * - after creating the pipeline targets, it'll determine the dependencies (the edges) from one target to another
 * - a pipeline "run(context)" call will provide some "entry targets", and then a subgraph will be generated to be submitted to the execution engine (p-graph)
 *
 * Distributed notes:
 * - for doing distributed work, the WrapperTask will instead place the PipelineTarget info into a worker queue
 */
class Pipeline {
    constructor(workspace, config) {
        this.workspace = workspace;
        this.config = config;
        this.targets = new Map([
            [
                exports.START_TARGET_ID,
                {
                    id: exports.START_TARGET_ID,
                    cwd: "",
                    run: () => { },
                    task: exports.START_TARGET_ID,
                },
            ],
        ]);
        this.dependencies = [];
        this.packageInfos = workspace.allPackages;
        this.graph = generateTopologicalGraph_1.generateTopologicGraph(workspace);
        this.loadConfig(config);
    }
    generatePackageTarget(packageName, task, deps) {
        const info = this.packageInfos[packageName];
        const id = taskId_1.getTargetId(packageName, task);
        return {
            id,
            task,
            cache: this.config.cache,
            outputGlob: this.config.cacheOptions.outputGlob,
            packageName: packageName,
            cwd: path_1.default.dirname(this.packageInfos[packageName].packageJsonPath),
            run: (args) => {
                const npmTask = new NpmScriptTask_1.NpmScriptTask(task, info, this.config, args.logger);
                return npmTask.run();
            },
            // TODO: do we need to really merge this? Is this desired? (this is the OLD behavior)
            deps: this.targets.has(id) ? [...(this.targets.get(id).deps || []), ...deps] : deps,
        };
    }
    /**
     * Expands the shorthand notation to pipeline targets (executable units)
     */
    expandShorthandTargets(id, deps) {
        // shorthand gets converted to npm tasks
        const { packageName, task } = taskId_1.getPackageAndTask(id);
        const results = [];
        let packagesWithScript = [];
        if (packageName) {
            // specific case in definition (e.g. 'package-name#test': ['build'])
            packagesWithScript.push(packageName);
        }
        else {
            // generic case in definition (e.g. 'test': ['build'])
            packagesWithScript = Object.entries(this.packageInfos)
                .filter(([_pkg, info]) => { var _a; return !!((_a = info.scripts) === null || _a === void 0 ? void 0 : _a[task]); })
                .map(([pkg, _info]) => pkg);
        }
        for (const packageWithScript of packagesWithScript) {
            results.push(this.generatePackageTarget(packageWithScript, task, deps));
        }
        return results;
    }
    /**
     * Given an id & factory, generate targets configurations
     * @param id
     * @param factory
     */
    generateFactoryTargets(factory) {
        const targets = factory({
            config: this.config,
            cwd: this.workspace.root,
        });
        return Array.isArray(targets) ? targets : [targets];
    }
    /**
     * Converts target configuration to pipeline target
     * @param id
     * @param target
     */
    convertToPipelineTarget(id, index, target) {
        if (target.type === "global") {
            return [
                Object.assign(Object.assign({}, target), { id: `${id}.${index}`, cache: target.cache !== false, cwd: this.workspace.root, task: id, run: target.run || (() => { }) }),
            ];
        }
        else {
            const packages = Object.keys(this.packageInfos);
            return packages.map((pkg) => (Object.assign(Object.assign({}, target), { id: taskId_1.getTargetId(pkg, id), cache: target.cache !== false, task: id, cwd: path_1.default.dirname(this.packageInfos[pkg].packageJsonPath), packageName: pkg, run: target.run || ((args) => new NpmScriptTask_1.NpmScriptTask(id, this.packageInfos[pkg], this.config, args.logger).run()) })));
        }
    }
    /**
     * Adds a target definition (takes in shorthand, target config, or a target config factory)
     * @param id
     * @param targetDefinition
     */
    addTargetDefinition(id, targetDefinition) {
        // e.g. build: ["^build", "prepare"]
        if (Array.isArray(targetDefinition)) {
            const targets = this.expandShorthandTargets(id, targetDefinition);
            for (const target of targets) {
                this.targets.set(target.id, target);
            }
        }
        else {
            // e.g. build: { /* target config */ }
            const targets = typeof targetDefinition === "function" ? this.generateFactoryTargets(targetDefinition) : [targetDefinition];
            targets.forEach((target, index) => {
                const pipelineTargets = this.convertToPipelineTarget(id, index, target);
                for (const pipelineTarget of pipelineTargets) {
                    this.targets.set(pipelineTarget.id, pipelineTarget);
                }
            });
        }
    }
    addDependencies() {
        const targets = [...this.targets.values()];
        for (const target of targets) {
            const { deps, packageName, id } = target;
            this.dependencies.push([exports.START_TARGET_ID, id]);
            if (!deps || deps.length === 0) {
                continue;
            }
            for (const dep of deps) {
                if (dep.includes("#")) {
                    // package and task as deps
                    this.dependencies.push([dep, id]);
                }
                else if (dep.startsWith("^") && packageName) {
                    // topo dep -> build: ['^build']
                    const depTask = dep.substr(1);
                    const dependencyIds = targets
                        .filter((needle) => {
                        const { task, packageName: needlePackageName } = needle;
                        return (task === depTask && this.graph[packageName].dependencies.some((depPkg) => depPkg === needlePackageName));
                    })
                        .map((needle) => needle.id);
                    for (const dependencyId of dependencyIds) {
                        this.dependencies.push([dependencyId, id]);
                    }
                }
                else if (packageName) {
                    // Intra package task dependency - only add the target dependency if it exists in the pipeline targets list
                    if (this.targets.has(taskId_1.getTargetId(packageName, dep))) {
                        this.dependencies.push([taskId_1.getTargetId(packageName, dep), target.id]);
                    }
                }
                else if (!dep.startsWith("^")) {
                    const dependencyIds = targets.filter((needle) => needle.task === dep).map((needle) => needle.id);
                    for (const dependencyId of dependencyIds) {
                        this.dependencies.push([dependencyId, id]);
                    }
                }
                else {
                    throw new Error(`invalid pipeline config detected: ${target.id}`);
                }
            }
        }
    }
    generateTargetGraph() {
        const scope = getPipelinePackages_1.getPipelinePackages(this.workspace, this.config);
        const tasks = this.config.command;
        const targetGraph = [];
        const queue = [];
        for (const task of tasks) {
            // package task
            for (const pkg of scope) {
                if (this.targets.has(taskId_1.getTargetId(pkg, task))) {
                    queue.push(taskId_1.getTargetId(pkg, task));
                    targetGraph.push([exports.START_TARGET_ID, taskId_1.getTargetId(pkg, task)]);
                }
            }
            // if we have globals, send those into the queue
            for (const target of this.targets.values()) {
                if (target.task === task && !target.packageName) {
                    queue.push(target.id);
                    targetGraph.push([exports.START_TARGET_ID, target.id]);
                }
            }
        }
        const visited = new Set();
        while (queue.length > 0) {
            const id = queue.shift();
            if (visited.has(id)) {
                continue;
            }
            visited.add(id);
            const { packageName, task } = taskId_1.getPackageAndTask(id);
            if (!packageName) {
                // global - find all deps in the form of "task.index"
                for (const [from, to] of this.dependencies) {
                    if (to.includes(".")) {
                        const toTaskName = to.split(".")[0];
                        if (toTaskName === task) {
                            targetGraph.push([from, to]);
                            if (from) {
                                queue.push(from);
                            }
                        }
                    }
                }
            }
            else {
                // package dep
                for (const [from, to] of this.dependencies) {
                    if (to === id) {
                        targetGraph.push([from, to]);
                        if (from) {
                            queue.push(from);
                        }
                    }
                }
            }
        }
        return targetGraph;
    }
    loadConfig(config) {
        this.config = config;
        for (const [id, targetDefinition] of Object.entries(this.config.pipeline)) {
            this.addTargetDefinition(id, targetDefinition);
        }
        // add target definitions for unknown tasks
        const knownTasks = new Set();
        for (const target of this.targets.values()) {
            knownTasks.add(target.task);
        }
        knownTasks.add(exports.START_TARGET_ID);
        const unknownCommands = this.config.command.filter((cmd) => !knownTasks.has(cmd));
        for (const command of unknownCommands) {
            this.addTargetDefinition(command, [`^${command}`]);
        }
        this.addDependencies();
    }
    getTargetPriority(target) {
        var _a, _b;
        return target.priority !== undefined
            ? target.priority
            : (_b = (_a = this.config.priorities) === null || _a === void 0 ? void 0 : _a.find((priority) => priority.package === target.packageName && priority.task === target.task)) === null || _b === void 0 ? void 0 : _b.priority;
    }
    async run(context) {
        const nodeMap = new Map();
        const targetGraph = this.generateTargetGraph();
        let redisClient;
        let workerQueue;
        if (this.config.dist) {
            const results = await workerQueue_1.initWorkerQueue(this.config.workerQueueOptions, false);
            redisClient = results.redisClient;
            workerQueue = results.workerQueue;
        }
        for (const [from, to] of targetGraph) {
            const fromTarget = this.targets.get(from);
            const toTarget = this.targets.get(to);
            for (const target of [fromTarget, toTarget]) {
                nodeMap.set(target.id, {
                    run: () => {
                        if (target.id === exports.START_TARGET_ID) {
                            return Promise.resolve();
                        }
                        if (!this.config.dist) {
                            const wrappedTask = new WrappedTarget_1.WrappedTarget(target, this.workspace.root, this.config, context);
                            return wrappedTask.run();
                        }
                        else {
                            const distributedTask = new DistributedTarget_1.DistributedTarget(target, this.config, context, workerQueue);
                            return distributedTask.run();
                        }
                    },
                    priority: this.getTargetPriority(target),
                });
            }
        }
        await p_graph_1.default(nodeMap, targetGraph).run({
            concurrency: this.config.concurrency,
            continue: this.config.continue,
        });
        if (redisClient) {
            redisClient.publish(workerQueue_1.workerPubSubChannel, "done");
            redisClient.quit();
        }
        if (workerQueue) {
            workerQueue.close();
        }
    }
}
exports.Pipeline = Pipeline;
