"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const TaskLogger_1 = require("../logger/TaskLogger");
const abortSignal_1 = require("./abortSignal");
const backfill_1 = require("../cache/backfill");
const formatDuration_1 = require("../logger/reporters/formatDuration");
const taskId_1 = require("./taskId");
class WrappedTask {
    constructor(target, root, config, context) {
        this.target = target;
        this.root = root;
        this.config = config;
        this.context = context;
        this.npmArgs = [];
        this.startTime = [0, 0];
        this.duration = [0, 0];
        this.status = "pending";
        this.logger = new TaskLogger_1.TaskLogger(target.packageName || "[GLOBAL]", target.packageName ? target.task : target.id);
    }
    onStart() {
        this.status = "started";
        this.startTime = process.hrtime();
        this.logger.info("started", { status: "started" });
    }
    onComplete() {
        this.status = "completed";
        this.duration = process.hrtime(this.startTime);
        this.logger.info("completed", {
            status: "completed",
            duration: formatDuration_1.hrToSeconds(this.duration),
        });
    }
    onFail() {
        this.status = "failed";
        this.duration = process.hrtime(this.startTime);
        this.logger.info("failed", {
            status: "failed",
            duration: formatDuration_1.hrToSeconds(this.duration),
        });
    }
    onSkipped(hash) {
        this.status = "skipped";
        this.duration = process.hrtime(this.startTime);
        this.logger.info(`skipped`, {
            status: "skipped",
            duration: formatDuration_1.hrToSeconds(this.duration),
            hash,
        });
    }
    async getCache() {
        let hash = null;
        let cacheHit = false;
        const { target, root, config } = this;
        if (config.cache) {
            hash = await backfill_1.cacheHash(target.id, target.cwd, root, config.cacheOptions, config.args);
            if (hash && !config.resetCache) {
                cacheHit = await backfill_1.cacheFetch(hash, target.id, target.cwd, config.cacheOptions);
            }
        }
        return { hash, cacheHit };
    }
    async saveCache(hash) {
        const { logger, config, target } = this;
        logger.verbose(`hash put ${hash}`);
        await backfill_1.cachePut(hash, target.cwd, config.cacheOptions);
    }
    async run() {
        const { target, context, config, logger } = this;
        try {
            const { hash, cacheHit } = await this.getCache();
            const cacheEnabled = target.cache && config.cache && hash;
            this.onStart();
            // skip if cache hit!
            if (cacheHit) {
                this.onSkipped(hash);
                return true;
            }
            if (cacheEnabled) {
                logger.verbose(`hash: ${hash}, cache hit? ${cacheHit}`);
            }
            // Wraps with profiler as well as task args
            await context.profiler.run(() => {
                let result;
                if (target.packageName) {
                    result = target.run({
                        packageName: target.packageName,
                        config: this.config,
                        cwd: target.cwd,
                        options: target.options,
                        taskName: taskId_1.getPackageAndTask(target.id).task,
                    });
                }
                else {
                    result = target.run({
                        config: this.config,
                        cwd: target.cwd,
                        options: target.options,
                    });
                }
                if (!result || typeof result["then"] !== "function") {
                    return Promise.resolve(result);
                }
                return result;
            }, target.id);
            if (cacheEnabled) {
                await this.saveCache(hash);
            }
            this.onComplete();
        }
        catch (e) {
            context.measures.failedTasks = context.measures.failedTasks || [];
            if (target.packageName) {
                context.measures.failedTasks.push({ pkg: target.packageName, task: target.task });
            }
            else {
                context.measures.failedTasks.push({ pkg: "[GLOBAL]", task: `${target.task} (${target.id})` });
            }
            this.onFail();
            if (config.continue) {
                return true;
            }
            if (!config.safeExit) {
                abortSignal_1.controller.abort();
            }
            return false;
        }
        return true;
    }
}
exports.WrappedTask = WrappedTask;
WrappedTask.npmCmd = "";
WrappedTask.activeProcesses = new Set();
WrappedTask.gracefulKillTimeout = 2500;
